/**
 * Representation of JWT Payload according to W3C spec.
 */
export declare abstract class JwtPayload {
    /**
     * Deserializes the payload from JWT format - Base64 encoded string.
     *
     * @param encoded - JWT encoded payload
     * @param mapFunction - function for mapping json to JwtPayload
     */
    static deserialize(encoded: string, mapFunction: (json: any) => JwtPayload): JwtPayload;
    iss: string;
    jti?: string;
    nbf: number;
    iat?: number;
    exp?: number;
    protected constructor(issuer: string, issuanceDate?: number, verifiableAttributeId?: string, expirationDate?: Date);
    /**
     * Serializes the payload into JWT format - Base64 encoded string.
     */
    serialize(): string;
    /**
     * Converts data to JSON for serialization.
     */
    protected abstract payloadToJSON(): any;
}
